CMD_APP_CREATE = 'create a new app'
OPT_APP_CREATE_DESCRIPTION = 'sets a description for your app'
OPT_APP_CREATE_POOL = 'defines which pool your app will be deployed'
OPT_APP_CREATE_TEAM = 'describes which team is responsible for the created app'
OPT_APP_CREATE_ROUTER = 'defines the router to be used'
OPT_APP_CREATE_PLAN = ' parameter defines the plan to be used'
OPT_APP_CREATE_TAG = 'sets a tag to your app. available multiple times'
OPT_APP_CREATE_DEPENDENCY = 'sets inspect file to create app dependencies. available multiple times'
OPT_APP_CREATE_ROUTER_opt = 'KEY=VALUE \n\n' \
                            'Allow passing custom parameters to the router used by the application\'s plan. ' \
                            'available multiple times'
CMD_APP_DEPLOY = 'deploy the source to shipa server'
OPT_APP_DEPLOY_APP = 'specify which app you want to deploy'
OPT_APP_DEPLOY_STEP_INTERVAL = 'single step duration. supported min: m, hour:h, second:s. ex. 1m, 60s, 1h'
OPT_APP_DEPLOY_STEPS = 'steps for canary deployment'
OPT_APP_DEPLOY_DIRECTORY = 'directory to deploy (default .)'
OPT_APP_DEPLOY_STEP_WEIGHT = 'deploy step weight'
CMD_APP_REMOVE = 'remove a shipa app'
OPT_APP_REMOVE_APP = 'specify which app you want to remove'
CMD_APP_MOVE = 'move a shipa app to another pool'
OPT_APP_MOVE_APP = 'specify which app you want to move'
OPT_APP_MOVE_POOL = 'specify which pool you want to move'
CMD_POOL_ADD = 'adds a new pool'
OPT_POOL_PUBLIC = 'make pool public (all teams can use it)'
OPT_POOL_DEFAULT = 'make pool default (when none is specified during app-create this pool will be used)'
OPT_POOL_PROVISIONER = 'provisioner associated to the pool (empty for default docker provisioner)'
OPT_POOL_PLAN = 'the plan when adding app to this pool'
OPT_POOL_QUOTA_LIMIT = 'quota limit for app when adding it to this pool'
OPT_POOL_NAMESPACE = 'kubernetes namespace for this pool'
OPT_POOL_DRIVER = 'node drivers accepted in pool. available multiple times'
CMD_POOL_REMOVE = 'remove an existing pool.'
CMD_POOL_UPDATE = 'updates attributes for a pool.'
CMD_NODE_ADD = 'creates or registers a new node in the cluster'
OPT_NODE_POOL = 'pool which will use this node. available multiple times'
OPT_NODE_REGISTER = 'registers an existing docker endpoint, the IaaS won\'t be called.'
OPT_NODE_CA_CERT = 'path to CA file shipa should trust.'
OPT_NODE_CLIENT_CERT = 'path to client TLS certificate file.'
OPT_NODE_CLIENT_KEY = 'path to client TLS key file.'
OPT_NODE_IAAS = 'which iaas provider should be used, if not set shipa will use the default ' \
                'iaas specified in shipa.conf file.'
OPT_NODE_IAASID = 'iaas\'s id.'
OPT_NODE_TEMPLATE = 'A machine template with predefined parameters, additional parameters will override ' \
                    'template ones.'
OPT_NODE_ADDRESS = 'only used if --register option is used. Should point to the endpoint of a working server.'
OPT_NODE_DRIVER = 'which node driver should be used. amazonec2, google, azure, vmwarevsphere, generic, ' \
                  'hyperv, openstack, virtualbox, vmwarefusion'
CMD_NODE_REMOVE = 'removes a node from the cluster'
CMD_NODE_DESTROY = 'destroy node from Iaas'
CMD_NODE_NO_REBALANCE = 'don\'t rebalance containers from removed node'
CMD_CLUSTER_ADD = 'creates a kubernetes cluster definition'
CMD_CLUSTER_ADDRESS = 'address to be used in cluster'
CMD_CLUSTER_TOKEN = 'token used for connection to the cluster'
OPT_CLUSTER_POOL = 'pool which will use this cluster'
OPT_CLUSTER_TEAM = 'cluster teams'
OPT_CLUSTER_DEFAULT = 'whether this is the default cluster.'
OPT_CLUSTER_INGRESS_SERVICE = 'specify ingress service type (LoadBalancer/ClusterIP/NodePort)'
OPT_CLUSTER_INGRESS_IP = 'ingress ip address'
OPT_CLUSTER_INGRESS_PORT = 'ingress port'
OPT_CLUSTER_CUSTOM = 'KEY=VALUE \n\n' \
                     'custom cluster specific data'
OPT_CLUSTER_PROVISIONER = 'represents a kubernetes cluster'
KUBERNETES_PROVISIONER = 'kubernetes'
OPT_CLUSTER_REMOVE = 'Removes a kubernetes cluster definition'
